module PollterGeist
  class IMAPCommands
    def initialize imap, logger
      @imap   = imap
      @logger = logger

      @idler = ImapIdler.new(self)
    end

    def select_mailbox mailbox
      logger.debug("selecting mailbox #{mailbox}")
      @imap.select(mailbox)
    end

    # Issues IDLE command for the imap server, it takes a list of events to listen for and returns once one of
    # events occured or if there was a timeout.
    #
    # @param events [events] a list of events to listen for, EXPUNGE, RECENT, EXISTS
    # @param timeout [Numeric] a timeout, if no event has ocoured for this time
    # @return will return an object which responds to #success?, #timeout? and #event
    def wait_for events, timeout = 29*60
      @idler.idle(events, timeout)
    end

    # fetches a list of message UIDs
    # @return [Array] of message UIDs
    def message_uids
      @logger.debug('fetching uids')
      # Searching for all messages where TO include @ is a workaround, since the imap#status method
      # does not work for some reason.
      message_ids = @imap.search(['TO', '@'])
      result      = @imap.fetch(message_ids, ['UID'])
      result.map { |r| r.attr['UID'].to_i }
    end

    def remove uid
      @logger.debug("removing #{uid}")
      @imap.uid_store uid, "+FLAGS", [:Deleted]
    end

    def expunge
      @logger.debug("expunge, remove all messages with Delete flag")
      @imap.expunge
    end

    def fetch uids
      logger.debug "fetching: #{uids}"
      @imap.uid_fetch(uids, ['RFC822']).map { |r| r.attr['RFC822'] }
    end

    def wait_for_changes timeout = 20*60
      logger.debug('waiting for changes in inbox')
      change_set = ChangeListener.new
      change_set.tick(message_uids)
      result = wait_for(['RECENT', 'EXPUNGE'], timeout)
      logger.debug('updating current view')
      change_set.tick(message_uids)
      if change_set.changed?
        logger.debug('changes detected!')
        yield(change_set)
      else
        logger.debug('no changes detected')
      end
    end

    def logger
      @logger
    end

    def imap
      @imap
    end
  end
end
