require 'logger'
require 'net/imap'

module PollterGeist
  class Poller
    LOGGER = Logger.new(STDOUT)

    def initialize options
      @options = options
    end

    def session &block
      start(&block)
    end

    private
    def start &block
      logger.debug "connecting to #{@options[:host]}"
      imap = Net::IMAP.new(@options[:host])
      logger.debug 'connected to imap server'

      imap.login(@options[:username], @options[:password])
      logger.debug 'authenticated to the server'

      begin
        commands = IMAPCommands.new(imap, logger)

        # preselecting mailbox if present in the options
        commands.select_mailbox(@options[:mailbox]) unless @options[:mailbox].nil?

        # let client code work their magic
        yield(commands)
      rescue Exception => e
        logger.error "#{e.class} : #{e.message} \n #{e.backtrace}"
        raise e
      ensure
        imap.logout
        logger.debug 'logoff'
      end
    end

    private
    def logger
      LOGGER
    end
  end
end
